import sys
from PyQt5.QtWidgets import (QApplication, QMainWindow, QScrollArea, QWidget,
                             QFormLayout, QLabel, QLineEdit, QComboBox, QPushButton, QCheckBox)
from PIL import Image, ImageDraw, ImageFont
import barcode
from barcode.writer import ImageWriter
import qrcode
from datetime import datetime

class WaybillGenerator(QMainWindow):
    def __init__(self):
        super().__init__()
        self.initUI()

    def initUI(self):
        self.setWindowTitle('快递面单生成器')
        self.setGeometry(100, 100, 800, 600)

        # 创建滚动区域
        scroll = QScrollArea()
        self.setCentralWidget(scroll)
        content = QWidget()
        scroll.setWidget(content)
        scroll.setWidgetResizable(True)
        layout = QFormLayout(content)

        # 主运单区字段
        layout.addRow(QLabel("<b>主运单区</b>"))
        self.proCode = QComboBox()
        self.proCode.addItems(['特快', '标快', '同城标快', 'T1', 'T4', 'T6', 'T8', 'T77', 'T68', 'ZT61', 'ZT77'])
        layout.addRow('时效类型:', self.proCode)

        self.waybillNo = QLineEdit()
        layout.addRow('运单号:', self.waybillNo)

        self.isMultiPiece = QCheckBox('一票多件')
        layout.addRow(self.isMultiPiece)
        self.subWaybillNo = QLineEdit()
        layout.addRow('子单号（如适用）:', self.subWaybillNo)
        self.pieceCount = QLineEdit()
        layout.addRow('件数（如适用）:', self.pieceCount)

        self.destRouteLabel = QLineEdit()
        layout.addRow('目的地（路由信息）:', self.destRouteLabel)

        self.codingMapping = QLineEdit()
        layout.addRow('进港映射码:', self.codingMapping)

        self.receiverName = QLineEdit()
        layout.addRow('收件人姓名:', self.receiverName)
        self.receiverPhone = QLineEdit()
        layout.addRow('收件人电话:', self.receiverPhone)
        self.receiverAddress = QLineEdit()
        layout.addRow('收件人地址:', self.receiverAddress)
        self.destAddrKeyWord = QLineEdit()
        layout.addRow('AOI关键字（可选）:', self.destAddrKeyWord)

        self.senderName = QLineEdit()
        layout.addRow('寄件人姓名:', self.senderName)
        self.senderPhone = QLineEdit()
        layout.addRow('寄件人电话:', self.senderPhone)
        self.senderAddress = QLineEdit()
        layout.addRow('寄件人地址:', self.senderAddress)
        self.senderCompany = QLineEdit()
        layout.addRow('寄件人公司名（可选）:', self.senderCompany)

        self.abFlag = QComboBox()
        self.abFlag.addItems(['', 'A', 'B'])
        layout.addRow('A/B标:', self.abFlag)

        self.codPod = QComboBox()
        self.codPod.addItems(['', 'COD', 'POD'])
        layout.addRow('代收货款/自取:', self.codPod)
        self.codAmount = QLineEdit()
        layout.addRow('代收金额（可选）:', self.codAmount)

        self.proName = QLineEdit()
        layout.addRow('产品名称:', self.proName)

        self.cargo = QLineEdit()
        layout.addRow('托寄物（可选）:', self.cargo)

        self.codingMappingOut = QLineEdit()
        layout.addRow('出港映射码（可选）:', self.codingMappingOut)

        self.destStationCode = QLineEdit()
        layout.addRow('目的站编码（可选）:', self.destStationCode)
        self.destTeamCode = QLineEdit()
        layout.addRow('配送编码（可选）:', self.destTeamCode)

        self.xbFlag = QCheckBox('顺心/退/转/逆/揽')
        layout.addRow(self.xbFlag)

        # 存根联区字段
        layout.addRow(QLabel("<b>存根联区</b>"))
        self.stubSenderName = QLineEdit()
        layout.addRow('寄件人姓名:', self.stubSenderName)
        self.stubSenderAddress = QLineEdit()
        layout.addRow('寄件人地址:', self.stubSenderAddress)
        self.stubSenderPhone = QLineEdit()
        layout.addRow('寄件人电话:', self.stubSenderPhone)

        self.stubReceiverName = QLineEdit()
        layout.addRow('收件人姓名:', self.stubReceiverName)
        self.stubReceiverAddress = QLineEdit()
        layout.addRow('收件人地址:', self.stubReceiverAddress)
        self.stubReceiverPhone = QLineEdit()
        layout.addRow('收件人电话:', self.stubReceiverPhone)

        self.stubCargo = QLineEdit()
        layout.addRow('托寄物:', self.stubCargo)
        self.paymentMethod = QComboBox()
        self.paymentMethod.addItems(['寄付', '到付'])
        layout.addRow('付款方式:', self.paymentMethod)

        self.stubPieces = QLineEdit()
        layout.addRow('件数:', self.stubPieces)
        self.stubWeight = QLineEdit()
        layout.addRow('重量:', self.stubWeight)

        # 生成按钮
        self.generateBtn = QPushButton('生成面单')
        self.generateBtn.clicked.connect(self.generateWaybill)
        layout.addRow(self.generateBtn)

    def desensitize_phone(self, phone):
        """电话号码脱敏：保留首位和后四位，其余用 * 替换"""
        if phone.startswith(('400', '800')):
            return phone  # 400/800 电话不脱敏
        if len(phone) < 5:
            return phone
        return phone[0] + '*' * (len(phone) - 5) + phone[-4:]

    def generateBarcode(self, waybillNo):
        """生成条码"""
        BARCODE = barcode.get_barcode_class('code128')
        barcode_img = BARCODE(waybillNo, writer=ImageWriter())
        return barcode_img.render()

    def generateQRCode(self, waybillNo, proCode, destRouteLabel):
        """生成二维码"""
        qr = qrcode.QRCode(version=1, box_size=10, border=4)
        k1, k2 = ' ', destRouteLabel if destRouteLabel else ' '
        k3, k4, k5 = ' ', proCode, waybillNo
        k6, k7 = self.abFlag.currentText(), ' '
        data = f'MMM={{"k1":"{k1}","k2":"{k2}","k3":"{k3}","k4":"{k4}","k5":"{k5}","k6":"{k6}","k7":"{k7}"}}'
        qr.add_data(data.ljust(128))  # 填充至 128 位
        qr.make(fit=True)
        return qr.make_image(fill_color="black", back_color="white")

    def generateWaybill(self):
        """生成面单图片"""
        # 收集输入信息
        proCode = self.proCode.currentText()
        waybillNo = self.waybillNo.text()
        isMultiPiece = self.isMultiPiece.isChecked()
        subWaybillNo = self.subWaybillNo.text() if isMultiPiece else ''
        pieceCount = self.pieceCount.text() if isMultiPiece else '1'
        destRouteLabel = self.destRouteLabel.text()
        codingMapping = self.codingMapping.text()
        receiverName = self.receiverName.text()
        receiverPhone = self.desensitize_phone(self.receiverPhone.text())
        receiverAddress = self.receiverAddress.text()
        destAddrKeyWord = self.destAddrKeyWord.text()
        senderName = self.senderName.text()
        senderPhone = self.desensitize_phone(self.senderPhone.text())
        senderAddress = self.senderAddress.text()
        senderCompany = self.senderCompany.text()
        abFlag = self.abFlag.currentText()
        codPod = self.codPod.currentText()
        codAmount = self.codAmount.text()
        proName = self.proName.text()
        cargo = self.cargo.text()
        codingMappingOut = self.codingMappingOut.text()
        destStationCode = self.destStationCode.text()
        destTeamCode = self.destTeamCode.text()
        xbFlag = self.xbFlag.isChecked()

        # 创建图片 (76mm x 130mm, 300 DPI)
        width, height = 897, 1535
        img = Image.new('RGB', (width, height), color='white')
        draw = ImageDraw.Draw(img)

        # 加载字体（需替换为实际字体路径）
        font_hei_26 = ImageFont.truetype('simhei.ttf', 26)
        font_hei_30 = ImageFont.truetype('simhei.ttf', 30)
        font_hei_40 = ImageFont.truetype('simhei.ttf', 40)
        font_song_6 = ImageFont.truetype('simsun.ttc', 6)
        font_song_8 = ImageFont.truetype('simsun.ttc', 8)
        font_song_9 = ImageFont.truetype('simsun.ttc', 9)
        font_song_14 = ImageFont.truetype('simsun.ttc', 14)
        font_song_22 = ImageFont.truetype('simsun.ttc', 22)
        font_arial_9 = ImageFont.truetype('arial.ttf', 9)

        # 主运单区布局
        x_left = 50  # 左边距
        x_right = width - 50  # 右边距
        y_offset = 50  # 初始 y 坐标

        # 时效类型（右上角）
        draw.text((x_right - 100, y_offset), proCode, font=font_hei_26, fill='black')
        y_offset += 40

        # 条码区（居中）
        barcode_img = self.generateBarcode(subWaybillNo if subWaybillNo else waybillNo)
        barcode_img = barcode_img.resize((int(66 * 300 / 25.4), int(13 * 300 / 25.4)))
        barcode_x = (width - barcode_img.width) // 2
        img.paste(barcode_img, (barcode_x, y_offset))
        if isMultiPiece:
            draw.text((x_left, y_offset + barcode_img.height + 10),
                      f"子单: {subWaybillNo} 母单: {waybillNo} 件数: {pieceCount}",
                      font=font_hei_30, fill='black')
        y_offset += barcode_img.height + 50

        # 目的地
        font_dest = font_hei_30 if len(destRouteLabel) <= 11 else ImageFont.truetype('simhei.ttf', 22)
        draw.text((x_left, y_offset), destRouteLabel, font=font_dest, fill='black')
        y_offset += 40

        # 进港映射码
        if codingMapping:
            draw.text((x_left, y_offset), codingMapping, font=font_hei_40, fill='black')
        y_offset += 50

        # 二维码
        qr_img = self.generateQRCode(subWaybillNo if subWaybillNo else waybillNo, proCode, destRouteLabel)
        qr_img = qr_img.resize((int(25 * 300 / 25.4), int(25 * 300 / 25.4)))
        img.paste(qr_img, (x_left, y_offset))
        y_offset += qr_img.height + 20

        # 收方信息
        receiver_text = f"{receiverName} {receiverPhone}\n{receiverAddress}{'【' + destAddrKeyWord + '】' if destAddrKeyWord else ''}"
        draw.text((x_left, y_offset), receiver_text, font=font_song_9, fill='black')
        y_offset += 60

        # 寄方信息
        sender_text = f"{senderName} {senderPhone}\n{senderAddress}\n{senderCompany}"
        draw.text((x_left, y_offset), sender_text, font=font_song_6, fill='black')
        y_offset += 60

        # A标和COD/POD
        if abFlag:
            draw.text((x_right - 50, y_offset), abFlag, font=font_song_14, fill='black')
        if codPod:
            draw.text((x_left, y_offset), codPod, font=font_song_14, fill='black')
        if codAmount:
            draw.text((x_left, y_offset + 20), f"代收金额: {codAmount}", font=font_song_6, fill='black')
        y_offset += 40

        # 已验视+打印时间
        draw.text((x_left, y_offset), f"已验视 {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}", font=font_song_6,
                  fill='black')
        y_offset += 20

        # 产品名称
        draw.text((x_left, y_offset), proName, font=font_song_14, fill='black')
        y_offset += 20

        # 托寄物
        if cargo:
            draw.text((x_left, y_offset), cargo, font=font_song_8, fill='black')
        y_offset += 20

        # 出港映射码
        if codingMappingOut:
            draw.text((x_left, y_offset), codingMappingOut, font=font_song_22, fill='black')
        y_offset += 30

        # 四周运单号
        draw.text((10, 10), waybillNo, font=font_song_8, fill='black')
        draw.text((width - 10, 10), waybillNo, font=font_song_8, fill='black', anchor='rt')
        draw.text((10, height - 10), waybillNo, font=font_song_8, fill='black', anchor='lb')
        draw.text((width - 10, height - 10), waybillNo, font=font_song_8, fill='black', anchor='rb')

        # 快运三段码
        if destStationCode:
            three_segment = f"{destStationCode}-{destTeamCode}" if destTeamCode else destStationCode
            draw.text((x_left, y_offset), three_segment, font=font_hei_30, fill='black')
        y_offset += 40

        # 存根联区（下半部分）
        stub_y_offset = height // 2  # 从图片中间开始
        draw.line([(0, stub_y_offset), (width, stub_y_offset)], fill='black', width=2)  # 分隔线
        stub_y_offset += 20
        draw.text((x_left, stub_y_offset),
                  f"寄: {self.stubSenderName.text()} {self.desensitize_phone(self.stubSenderPhone.text())}\n{self.stubSenderAddress.text()}",
                  font=font_hei_30, fill='black')
        stub_y_offset += 40
        draw.text((x_left, stub_y_offset),
                  f"收: {self.stubReceiverName.text()} {self.desensitize_phone(self.stubReceiverPhone.text())}\n{self.stubReceiverAddress.text()}",
                  font=font_hei_30, fill='black')
        stub_y_offset += 40
        draw.text((x_left, stub_y_offset), f"托寄物: {self.stubCargo.text()}", font=font_hei_30, fill='black')
        stub_y_offset += 20
        draw.text((x_left, stub_y_offset), f"付款方式: {self.paymentMethod.currentText()}", font=font_hei_30,
                  fill='black')
        stub_y_offset += 20
        draw.text((x_left, stub_y_offset), f"件数: {self.stubPieces.text()} 重量: {self.stubWeight.text()}",
                  font=font_hei_30, fill='black')

        # 保存图片
        img.save('waybill.png')
        print("面单已生成：waybill.png")


        
if __name__ == '__main__':
    app = QApplication(sys.argv)
    ex = WaybillGenerator()
    ex.show()
    sys.exit(app.exec_())